/*
 * MainFrame.java
 *
 * Created on 9 de marzo de 2007, 14:21
 */

package datareader;

import com.coti.tools.*;
import java.awt.Dimension;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.table.TableColumnModel;

/**
 *
 * @author  bruegel
 */
public class MainFrame extends javax.swing.JFrame {
    
    /** Creates new form MainFrame */
    public MainFrame() {
        initComponents();
        rp = new StringTableView(this);
        this.setResizable(false);
        this.getContentPane().add(rp);
        this.pack();
        datafile = null;
        dataChanged = false;
        jfc = new JFileChooser(System.getProperty("user.dir"));
        at = new AuxText(this);
        shouldQuit = false;
        saveItem.setEnabled(false);
        saveAsItem.setEnabled(false);
        closeItem.setEnabled(false);
    }
    
    private void initComponents() {
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        openItem = new javax.swing.JMenuItem();
        saveItem = new javax.swing.JMenuItem();
        saveAsItem = new javax.swing.JMenuItem();
        closeItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        quitItem = new javax.swing.JMenuItem();
        
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        jMenu1.setText("File");
        openItem.setText("Open...");
        openItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openItemActionPerformed(evt);
            }
        });
        
        jMenu1.add(openItem);
        
        saveItem.setText("Save");
        saveItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveItemActionPerformed(evt);
            }
        });
        
        jMenu1.add(saveItem);
        
        saveAsItem.setText("Save as...");
        saveAsItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveAsItemActionPerformed(evt);
            }
        });
        
        jMenu1.add(saveAsItem);
        
        closeItem.setText("Close");
        closeItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeItemActionPerformed(evt);
            }
        });
        
        jMenu1.add(closeItem);
        
        jMenu1.add(jSeparator1);
        
        quitItem.setText("Quit");
        quitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitItemActionPerformed(evt);
            }
        });
        
        jMenu1.add(quitItem);
        
        jMenuBar1.add(jMenu1);
        
        setJMenuBar(jMenuBar1);
    }
    
    
    private void quitItemActionPerformed(java.awt.event.ActionEvent evt) {
        shouldQuit = true;
        if(dataChanged){
            int answer = JOptionPane.showConfirmDialog(this,
                    "Do you want to save changes?",
                    "Data have been edited",
                    JOptionPane.YES_NO_CANCEL_OPTION);
            switch(answer){
                case JOptionPane.YES_OPTION:
                    saveAsItemActionPerformed(evt);
                    break;
                case JOptionPane.NO_OPTION:
                    break;
                case JOptionPane.CANCEL_OPTION:
                    shouldQuit = false;
                    break;
            }
        }
        if (shouldQuit)
            System.exit(0);
    }
    
    private void closeItemActionPerformed(java.awt.event.ActionEvent evt) {
        shouldQuit = false;
        if(dataChanged){
            int answer = JOptionPane.showConfirmDialog(this,
                    "Do you want to save changes?",
                    "Data have been edited",
                    JOptionPane.YES_NO_CANCEL_OPTION);
            switch(answer){
                case JOptionPane.YES_OPTION:
                    saveAsItemActionPerformed(evt);
                    break;
                case JOptionPane.NO_OPTION:
                case JOptionPane.CANCEL_OPTION:
                    break;
            }
        }
        
        rp.getStm().clearModel();
        rp.getJTable1().createDefaultColumnsFromModel();
        openItem.setEnabled(true);
        saveItem.setEnabled(false);
        saveAsItem.setEnabled(false);
        closeItem.setEnabled(false);
    }
    
    private void saveAsItemActionPerformed(java.awt.event.ActionEvent evt) {
        File outputfile = null;
        int result = jfc.showSaveDialog(this);
        String[][] data = rp.getStm().getDataAsStringTable();
        switch(result){
            case JFileChooser.APPROVE_OPTION:
                outputfile = jfc.getSelectedFile();
                if (!outputfile.exists()){
                    at.writeDelimitedTable(data,"\t",outputfile);
                    JOptionPane.showMessageDialog(this,datafile.getName()+" was written correctly.","Confirmation",JOptionPane.INFORMATION_MESSAGE);
                    if (shouldQuit)
                        System.exit(0);
                } else {
                    int answer = JOptionPane.showConfirmDialog(this,
                            "Do you want to overwrite it?",
                            "File already exists",
                            JOptionPane.YES_NO_OPTION);
                    switch(answer){
                        case JOptionPane.YES_OPTION:
                            at.writeDelimitedTable(data,"\t",outputfile);
                            JOptionPane.showMessageDialog(this,datafile.getName()+" was written correctly.","Confirmation",JOptionPane.INFORMATION_MESSAGE);
                            if (shouldQuit)
                                System.exit(0);
                            break;
                        case JOptionPane.NO_OPTION:
                            shouldQuit = false;
                            break;
                    }
                }
                break;
            case JFileChooser.CANCEL_OPTION:
                shouldQuit = false;
                break;
        }
    }
    
    private void saveItemActionPerformed(java.awt.event.ActionEvent evt) {
        String[][] data = rp.getStm().getDataAsStringTable();
        at.writeDelimitedTable(data,"\t",datafile);
        JOptionPane.showMessageDialog(  this,   datafile.getName()
        +" was written correctly.",
                "Confirmation",
                JOptionPane.INFORMATION_MESSAGE);
    }
    
    private void openItemActionPerformed(java.awt.event.ActionEvent evt) {
        int result = jfc.showOpenDialog(this);
        StringTableModel stm = rp.getStm();
        switch(result){
            case JFileChooser.APPROVE_OPTION:
                datafile = jfc.getSelectedFile();
                if (dataChanged)
                    closeItemActionPerformed(evt);
                String[][] table = at.readDelimitedTable("\t",datafile);
                stm.clearModel();
                stm.populateWithTable(table);
                rp.setStm(stm);
                rp.getJTable1().setModel(stm);
                rp.getJTable1().createDefaultColumnsFromModel();
                TableColumnModel cm = rp.getJTable1().getColumnModel();
                for(int i=0;i<table[0].length;i++)
                    cm.getColumn(i).setPreferredWidth(100);
                rp.getJTable1().revalidate();
                saveAsItem.setEnabled(true);
                closeItem.setEnabled(true);
                saveItem.setEnabled(false);
                break;
            case JFileChooser.CANCEL_OPTION:
                break;
        }
        
    }
    
    private javax.swing.JMenuItem closeItem;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenuItem openItem;
    private javax.swing.JMenuItem quitItem;
    private javax.swing.JMenuItem saveAsItem;
    private javax.swing.JMenuItem saveItem;
    
    private StringTableView rp;
    private File datafile;
    private boolean dataChanged;
    private AuxText at;
    private JFileChooser jfc;
    private boolean shouldQuit;
    
    public StringTableView getRp() {
        return rp;
    }
    
    public void setRp(StringTableView rp) {
        this.rp = rp;
    }

    public boolean isDataChanged() {
        return dataChanged;
    }

    public void setDataChanged(boolean dataChanged) {
        this.dataChanged = dataChanged;
    }
}
