import java.awt.Color;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Scanner;
import javax.swing.JApplet;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import com.coti.graphics.BarPlot;

/**
 *
 * @author bruegel
 */
 /*
  Note
  ====
  
  The first row of the data file must contain a tab-separated list
  that consists of precisely three strings:
    - The title of the plot
    - The X axe legend
    - The Y axe legend
*/
public class BarPlotApplet extends JApplet implements Runnable {
    
    /*
        List of methods
     */
    public void init(){
        initComponents();
    }
    
    public void initComponents(){
        jta = new JTextArea();
        jta.setLineWrap(true);
        jta.setWrapStyleWord(true);
        jsc = new JScrollPane(jta);
        jt = new JTabbedPane();
        jt.add(jsc,"Data");
        add(jt);
        loadFinished=false;
        dataLoaded=false;
        x = null;
        y = null;
        doLoad();
        try {
            while(!loadFinished) Thread.sleep(100);
        } catch(Exception e) {
        }
        if (dataLoaded) {
        		double[][] data = new double[x.length][2];
        		for(int i=0;i<data.length;i++)
        			{
        				data[i][0] = x[i];
        				data[i][1] = y[i];
        			}
            BarPlot p = new BarPlot(data,legend);
            jt.add(p,"Bar plot");
        }
    }
    
    public void run() {
        rawdata.clear();
        jta.setText("Loading data...");
        try {
            URL u = new URL(getCodeBase(),"data.txt");
            URLConnection uc = u.openConnection();
            Scanner sc = new Scanner(uc.getInputStream());
            while(sc.hasNextLine())
                rawdata.add(sc.nextLine());
            sc.close();
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
            jta.setText("");
            putText("\nError:\n"+ex);
            loadFinished=true;
            dataLoaded=false;
            return;
        }catch (java.io.IOException ex) {
            jta.setText("");
            putText("Error:\n"+ex);
            ex.printStackTrace();
            loadFinished=true;
            dataLoaded=false;
            return;
        } catch(Exception ex){
            putText("Error:\n"+ex);
            ex.printStackTrace();
            loadFinished=true;
            dataLoaded=false;
            return;
        }
        
        int numberOfRows = rawdata.size();
        x = new float[numberOfRows-1];
        y = new float[numberOfRows-1];
        String[] fields = null;
        try {
            // Row 0 is supposed to contain header information
            legend = rawdata.get(0).split("\t");
            // Rows 1 till last are supposed to contain numbers
            for(int i=1;i<numberOfRows;i++){
                fields = rawdata.get(i).split("\t");
                x[i-1] = Float.parseFloat(fields[0].trim());
                y[i-1] = Float.parseFloat(fields[1].trim());
            }
            putText("done.\n\nData are as follows:\n\n");
            for(String s:rawdata)
                putText(s+"\n");
            putText("\n\n----End of data----\n");
            dataLoaded=true;
            loadFinished=true;
        } catch(Exception e) {
            putText("***Incorrect data found***\n"+e);
            loadFinished=true;
            dataLoaded=false;
        }
    }
    
    void putText(final String text) {
        SwingUtilities.invokeLater( new Runnable() {
            public void run() {
                jta.append(text);
                jta.setCaretPosition(text.length());
            }
        } );
    }
    
    private void doLoad() {
        Thread t = new Thread(this);
        t.start();
    }
    
    /*
        List of attributes
     */
    ArrayList<String> rawdata = new ArrayList<String>();
    
    private JTabbedPane jt;
    private JTextArea jta;
    private JScrollPane jsc;
    private float[] x;
    private float[] y;
    private String[] legend;
    private boolean loadFinished,dataLoaded;
}
