import java.awt.*;       
import java.awt.event.*; 
import javax.swing.*;    
import java.io.*;        
import java.net.*;
import java.util.*;

public class MyReadURLApplet extends JApplet 
                 implements Runnable, ActionListener {

   JTextArea textDisplay;  // data loaded from url is displayed here
   JTextField inputBox;    // user enters name of url to be loaded here
   JButton loadButton;     // user clicks on this button to load the url
   
   String urlName;  	// the name of the url to be loaded by the run() method.
							       	// Note:  this url can be specified relative to the
       								//        HTML document that contains the applet.
   
   Thread loader;   	// thread for reading data from the url

   
   public void init() {         
      setBackground(Color.gray);
      getContentPane().setBackground(Color.gray);
      
      textDisplay = new JTextArea();
      textDisplay.setEditable(false);
      textDisplay.setBackground(Color.white);
      
      loadButton = new JButton("Load");
      loadButton.addActionListener(this);
      
      String url = getParameter("URL");
      if (url == null) 
         inputBox = new JTextField();
      else
         inputBox = new JTextField(url);
      inputBox.setBackground(Color.white);
      inputBox.addActionListener(this);
      
      JPanel bottom = new JPanel();
      bottom.setBackground(Color.gray);
      bottom.setLayout(new BorderLayout(3,3));
      bottom.add(inputBox, BorderLayout.CENTER);
      bottom.add(loadButton, BorderLayout.EAST);
      bottom.add(new JLabel("Enter URL:"), BorderLayout.WEST);
      
      getContentPane().setLayout(new BorderLayout(3,3));
      getContentPane().add(new JScrollPane(textDisplay), BorderLayout.CENTER);
      getContentPane().add(bottom, BorderLayout.SOUTH);
      
      inputBox.requestFocus();

   }  // end init()
   
   
   public Insets getInsets() {
      return new Insets(3,3,3,3);
   }

   
   void doLoad(String urlToLoad) {
      inputBox.setEditable(false);
      loadButton.setEnabled(false);
      textDisplay.setText("Connecting...");
      urlName = urlToLoad;  // set the urlName which will be used by the thread
      loader = new Thread(this);
      loader.start();
   }
   
   
   public void actionPerformed(ActionEvent evt) {
         // responds when the user clicks on the load button or
         // presses return in the input box
      String urlToLoad = inputBox.getText().trim();
      if (urlToLoad.equals("")) {
         textDisplay.setText("No URL found in text-input box");
         inputBox.requestFocus();
      }
      else
         doLoad(urlToLoad);
   }
   
   
   void putText(final String text) {
        
      SwingUtilities.invokeLater( new Runnable() {
            public void run() {
               textDisplay.setText(text);
               textDisplay.setCaretPosition(text.length());
            }
         } );
   }

   public void run() {

      try {

         URL url = new URL(getDocumentBase(), urlName);

         URLConnection connection = url.openConnection();
         InputStream urlData = connection.getInputStream();

         /* Check that the content is some type of text. */

         String contentType = connection.getContentType();
         if (contentType == null || contentType.startsWith("text") == false)
            throw new IOException("URL does not refer to a text file.");
            
         putText("Receiving Data...");
            
         Scanner in; 
         in = new Scanner(urlData);
         ArrayList<String> data = new ArrayList<String>();
         while (in.hasNextLine()) {
            data.add(in.nextLine()+"\n");
         }
         in.close();
         
         /* Finally, put the text that has been received into the 
            text area. */
         textDisplay.setText("");
         for(String s : data)
					textDisplay.append(s);
         textDisplay.setCaretPosition(textDisplay.getText().length());
      }
      catch (MalformedURLException e) {
             // Can be thrown when URL is created.
         putText(
             "\nERROR!  Improper syntax given for the URL to be loaded.");
      }
      catch (SecurityException e) {  
             // Can be thrown when the connection is created.
         putText("\nSECURITY ERROR!  Can't access that URL.");
      }
      catch (IOException e) {  
             // Can be thrown while data is being read.
         putText(
                    "\nINPUT ERROR! Problem reading data from that URL:\n"
                     + e.toString());
      }
      finally { 
         loadButton.setEnabled(true);
         inputBox.setEditable(true);
         inputBox.selectAll();
         inputBox.requestFocus();
      }

   } // end of run() method

} // end of class

